#include "StdAfx.h"
#include "SynSerial.h"

CSynSerial::CSynSerial(void) :m_hSerialPort(NULL)
, m_dwInQueue(1024)
, m_dwOutQueue(1024)
{
	memset(&m_dcb, 0, sizeof(DCB));
	memset(&m_cts, 0, sizeof(COMMTIMEOUTS));
	memset(&m_szSerialPort, 0, sizeof(TCHAR)*MAX_PORT_LEN);
}

CSynSerial::~CSynSerial(void)
{
	CloseSerialPort();
}

BOOL CSynSerial::OpenSerialPort(IN CONST BYTE & byCommPort, IN CONST DWORD & dwBaudRate, IN CONST BYTE & bySize, IN CONST BYTE & byParity, IN CONST BYTE & byStopBits, IN CONST BYTE & byStartAddr, IN CONST INT & nInterval)
{
	ASSERT(byCommPort);
	CloseSerialPort();

	_stprintf_s(m_szSerialPort, _T("\\\\.\\com%d"), (int)byCommPort);
	m_hSerialPort = CreateFile(m_szSerialPort, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, NULL, NULL);
	if (m_hSerialPort == INVALID_HANDLE_VALUE)
	{
		// 򿪴ʧ;
		DWORD dwError = GetLastError();
		return FALSE;
	}

	BOOL bResult = FALSE;
	// SetupCommûС;
	bResult = SetupComm(m_hSerialPort, m_dwInQueue, m_dwOutQueue);
	if (!bResult)
	{
		DWORD dwError = GetLastError();
		return FALSE;
	}

	// ڴڶд֮ǰ,;
	bResult = PurgeComm(m_hSerialPort, PURGE_TXCLEAR | PURGE_RXCLEAR);
	if (!bResult)
	{
		DWORD dwError = GetLastError();
		return FALSE;
	}

	// GetCommStateȡ豸ƿ״̬;
	memset(&m_dcb, 0, sizeof(DCB));
	bResult = GetCommState(m_hSerialPort, &m_dcb);
	if (!bResult)
	{
		DWORD dwError = GetLastError();
		return FALSE;
	}

	m_dcb.Parity = byParity;
	if (m_dcb.Parity == NOPARITY)
		m_dcb.fParity = FALSE;
	else
		m_dcb.fParity = TRUE;

	m_dcb.BaudRate = dwBaudRate;
	m_dcb.ByteSize = bySize;
	m_dcb.StopBits = byStopBits;
	if (m_dcb.ByteSize == 8)
		m_dcb.StopBits = ONESTOPBIT;

	// SetCommState豸Ŀƿ״̬;
	memset(&m_cts, 0, sizeof(COMMTIMEOUTS));
	bResult = SetCommState(m_hSerialPort, &m_dcb);
	if (!bResult)
	{
		DWORD dwError = GetLastError();
		return FALSE;
	}

	// ȡ豸ĳʱֵ;
	bResult = GetCommTimeouts(m_hSerialPort, &m_cts);
	if (!bResult)
	{
		DWORD dwError = GetLastError();
		return FALSE;
	}

	// ַ֮ʱ,0ʾ;
	m_cts.ReadIntervalTimeout = 50;
	m_cts.ReadTotalTimeoutMultiplier = 100;
	m_cts.ReadTotalTimeoutConstant = 8000;
	m_cts.WriteTotalTimeoutMultiplier = 100;
	m_cts.WriteTotalTimeoutConstant = 5000;

	// 豸ĳʱֵ;
	bResult = SetCommTimeouts(m_hSerialPort, &m_cts);
	if (!bResult)
	{
		DWORD dwError = GetLastError();
		return FALSE;
	}

	return TRUE;
}

BOOL CSynSerial::ReOpenSerialPort()
{
	ASSERT(_tcslen(m_szSerialPort));
	CloseSerialPort();

	m_hSerialPort = CreateFile(m_szSerialPort, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, NULL, NULL);
	if (m_hSerialPort == INVALID_HANDLE_VALUE)
	{
		// 򿪴ʧ;
		DWORD dwError = GetLastError();
		return FALSE;
	}

	BOOL bResult = FALSE;
	// SetupCommûС;
	bResult = SetupComm(m_hSerialPort, m_dwInQueue, m_dwOutQueue);
	if (!bResult)
	{
		DWORD dwError = GetLastError();
		return FALSE;
	}

	// ڴڶд֮ǰ,;
	bResult = PurgeComm(m_hSerialPort, PURGE_TXCLEAR | PURGE_RXCLEAR);
	if (!bResult)
	{
		DWORD dwError = GetLastError();
		return FALSE;
	}

	// SetCommState豸Ŀƿ״̬;
	memset(&m_cts, 0, sizeof(COMMTIMEOUTS));
	bResult = SetCommState(m_hSerialPort, &m_dcb);
	if (!bResult)
	{
		DWORD dwError = GetLastError();
		return FALSE;
	}

	// 豸ĳʱֵ;
	bResult = SetCommTimeouts(m_hSerialPort, &m_cts);
	if (!bResult)
	{
		DWORD dwError = GetLastError();
		return FALSE;
	}

	return TRUE;
}

void CSynSerial::CloseSerialPort()
{
	if (m_hSerialPort != NULL && m_hSerialPort != INVALID_HANDLE_VALUE)
		CloseHandle(m_hSerialPort);
	m_hSerialPort = NULL;
}

BOOL CSynSerial::SetSerialPort(IN CONST DWORD & dwBaudRate, IN CONST BYTE & byByteSize, IN CONST BYTE & byParity, IN CONST BYTE & byStopBits, IN CONST BYTE & byStartAddr, IN CONST DWORD & dwInQueue, IN CONST DWORD & dwOutQueue)
{
	if (!IsOpen())
		return FALSE;

	BOOL bResult = FALSE;
	// GetCommStateȡ豸ƿ״̬;
	memset(&m_dcb, 0, sizeof(DCB));
	bResult = GetCommState(m_hSerialPort, &m_dcb);
	if (!bResult)
	{
		DWORD dwError = GetLastError();
		return FALSE;
	}

	m_dcb.Parity = byParity;
	if (m_dcb.Parity == NOPARITY)
		m_dcb.fParity = FALSE;
	else
		m_dcb.fParity = TRUE;

	m_dcb.BaudRate = dwBaudRate;
	m_dcb.ByteSize = byByteSize;
	m_dcb.StopBits = byStopBits;
	if (m_dcb.ByteSize == 8)
		m_dcb.StopBits = ONESTOPBIT;

	// SetCommState豸Ŀƿ״̬;
	memset(&m_cts, 0, sizeof(COMMTIMEOUTS));
	bResult = SetCommState(m_hSerialPort, &m_dcb);
	if (!bResult)
	{
		DWORD dwError = GetLastError();
		return FALSE;
	}

	// ȡ豸ĳʱֵ;
	bResult = GetCommTimeouts(m_hSerialPort, &m_cts);
	if (!bResult)
	{
		DWORD dwError = GetLastError();
		return FALSE;
	}

	// ַ֮ʱ,0ʾ;
	m_cts.ReadIntervalTimeout = 50;
	m_cts.ReadTotalTimeoutMultiplier = 100;
	m_cts.ReadTotalTimeoutConstant = 8000;
	m_cts.WriteTotalTimeoutMultiplier = 100;
	m_cts.WriteTotalTimeoutConstant = 3000;

	// 豸ĳʱֵ;
	bResult = SetCommTimeouts(m_hSerialPort, &m_cts);
	if (!bResult)
	{
		DWORD dwError = GetLastError();
		return FALSE;
	}

	return TRUE;
}

void CSynSerial::SetSerialDCB(IN CONST DCB & dcb)
{
}

void CSynSerial::SetSerialCommTimeouts(IN CONST COMMTIMEOUTS & commTimeouts)
{
}

DWORD CSynSerial::WriteComm(IN BYTE * pWirteBuf, IN CONST INT32 nWriteSize)
{
	if (!IsOpen())
		return 0;

	if (pWirteBuf == NULL || !::AfxIsValidAddress(pWirteBuf, nWriteSize, FALSE))
	{
		return 0;
	}
	
	DWORD dwErrorFlags;
	DWORD dwBytesWritten = 0;	// ʵдֽ;
	// дǰ, ;
	COMSTAT ComStat;
	ClearCommError(m_hSerialPort, &dwErrorFlags, &ComStat);	
	// дǰ,ջ;
	PurgeComm(m_hSerialPort, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR);
	if ( !WriteFile(m_hSerialPort, pWirteBuf, nWriteSize, &dwBytesWritten, NULL) )
	{
		DWORD dwError = GetLastError();
	}	

	// дֽ;
	return dwBytesWritten;
}

DWORD CSynSerial::ReadComm(IN BYTE *pReadBuf, IN CONST DWORD& dwReadSize, WORD timeout /* = 300 */)
{
	if (!IsOpen())
		return 0;

	if (pReadBuf == NULL || !::AfxIsValidAddress(pReadBuf, dwReadSize, FALSE))
	{
		return 0;
	}

	DWORD dwBytesRead = 0;
	DWORD dwErrorFlags = 0;
	// ReadFileǰʹClearCommError;
	COMSTAT ComStat = {0};
	DWORD dwLastLen = 0;
	ULONGLONG ulTick = GetTickCount64();	
	// ֱΪֹʱ3;
	while (true)
	{// cbInQueʾ뻺ֽ; 
		Sleep(timeout);
		if (GetTickCount64() - ulTick > 3000)
			break;
		ClearCommError(m_hSerialPort, &dwErrorFlags, &ComStat);
		// ֹһû꣬ٶһ;γһʾ;
		if (ComStat.cbInQue != 0 && dwLastLen == ComStat.cbInQue)
			break;
		dwLastLen = ComStat.cbInQue;
	}

	if (ComStat.cbInQue == 0)
	{
		// ڳʱݷ;
		return 0;
	}

	dwBytesRead = ComStat.cbInQue;
	if (dwReadSize < dwBytesRead)
	{
		dwBytesRead = dwReadSize;
	}

	if (ReadFile(m_hSerialPort, pReadBuf, dwBytesRead, &dwBytesRead, NULL))
	{
		return (INT)dwBytesRead;
	}

	return dwBytesRead;
}
